""" Set of functions to define the common rules for access control across collections
"""
import logging

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.workspace import api as workspace_api
from core_main_app.permissions import api as permissions_api, rights as rights
from core_main_app.settings import CAN_SET_PUBLIC_DATA_TO_PRIVATE, CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT

logger = logging.getLogger(__name__)


def has_perm_publish(user, codename):
    """ Does the user have the permission to publish.

    Args:
        user
        codename

    Returns
    """
    publish_perm = permissions_api.get_by_codename(codename)
    if not user.has_perm(publish_perm.content_type.app_label + '.' + publish_perm.codename):
        raise AccessControlError("The user doesn't have enough rights to publish.")


def has_perm_administration(func, *args, **kwargs):
    """ Is the given user has administration rights.

        Args:
            func:
            *args:
            **kwargs:

        Returns:

        """
    try:
        if args[0].is_superuser:
            return func(*args, **kwargs)
    except Exception as e:
        logger.warning("has_perm_administration threw an exception: ".format(str(e)))

    raise AccessControlError("The user doesn't have enough rights.")


def check_can_write(document, user):
    """ Check that the user can write.

    Args:
        document:
        user:

    Returns:

    """
    # TODO: data will inherit of workspace rights, which means a owner can't edit or delete a data if data in wkp that doesn't give hin write rights
    if hasattr(document, 'workspace') and document.workspace is not None:
        if workspace_api.is_workspace_public(document.workspace):
            has_perm_publish(user, rights.publish_data)
        else:
            _check_can_write_in_workspace(document.workspace, user)

    # not the owner and workspace is not set or None
    if document.user_id != str(user.id) and (not hasattr(document, 'workspace') or document.workspace is None):
        raise AccessControlError("The user doesn't have enough rights.")


def check_can_read_list(document_list, user):
    """ Check that the user can read each document of the list.

    Args:
        document_list:
        user:

    Returns:

    """
    if len(document_list) > 0:
        # get list of accessible workspaces
        accessible_workspaces = workspace_api.get_all_workspaces_with_read_access_by_user(user)
        # check access is correct
        for document in document_list:
            # user is document owner
            if document.user_id == str(user.id):
                continue
            # user is not owner or document not in accessible workspace
            if document.workspace is None or document.workspace not in accessible_workspaces:
                raise AccessControlError("The user doesn't have enough rights.")


def can_write_document_in_workspace(func, document, workspace, user):
    """ Can user write data in workspace.

    Args:
        func:
        document:
        workspace:
        user:

    Returns:

    """
    return can_write_in_workspace(func, document, workspace, user, rights.publish_data)


def can_read_or_write_in_workspace(func, workspace, user):
    """ Can user read or write in workspace.

    Args:
        func:
        workspace:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(workspace, user)

    _check_can_read_or_write_in_workspace(workspace, user)
    return func(workspace, user)


def can_write_in_workspace(func, document, workspace, user, codename):
    """ Can user write in workspace.

    Args:
        func:
        document:
        workspace:
        user:
        codename:

    Returns:

    """
    if user.is_superuser:
        return func(document, workspace, user)
    if workspace is not None:
        if workspace_api.is_workspace_public(workspace):
            has_perm_publish(user, codename)
        else:
            _check_can_write_in_workspace(workspace, user)

    check_can_write(document, user)

    # if we can not unpublish
    if CAN_SET_PUBLIC_DATA_TO_PRIVATE is False:
        # if document is in public workspace
        if document.workspace is not None and workspace_api.is_workspace_public(document.workspace):
            # if target workspace is private
            if workspace is None or workspace_api.is_workspace_public(workspace) is False:
                raise AccessControlError("The document can not be unpublished.")

    return func(document, workspace, user)


def can_read(func, user):
    """ Can a user read

    Args:
        func:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(user)

    # get list of document
    document_list = func(user)
    # check that the user can access the list of document
    check_can_read_list(document_list, user)
    # return list of document
    return document_list


def can_read_id(func, document_id, user):
    """ Can read from object id.

    Args:
        func:
        document_id:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(document_id, user)

    document = func(document_id, user)
    _check_can_read(document, user)
    return document


def can_write(func, document, user):
    """ Can user write

    Args:
        func:
        document:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(document, user)

    check_can_write(document, user)
    return func(document, user)


def _check_can_write_in_workspace(workspace, user):
    """ Check that user can write in the workspace.

    Args:
        workspace:
        user:

    Returns:

    """
    accessible_workspaces = workspace_api.get_all_workspaces_with_write_access_by_user(user)
    if workspace not in accessible_workspaces:
        raise AccessControlError("The user does not have the permission to write into this workspace.")


def _check_can_read_or_write_in_workspace(workspace, user):
    """ Check that user can read or write in the workspace.

    Args:
        workspace:
        user:

    Returns:

    """
    accessible_write_workspaces = workspace_api.get_all_workspaces_with_write_access_by_user(user)
    accessible_read_workspaces = workspace_api.get_all_workspaces_with_read_access_by_user(user)
    if workspace not in list(accessible_write_workspaces) + list(accessible_read_workspaces):
        raise AccessControlError("The user does not have the permission to read or write into this workspace.")


def _check_can_read(document, user):
    """ Check that the user can read.

    Args:
        document:
        user:

    Returns:

    """
    # anonymous user case
    if user.is_anonymous and not CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
        raise AccessControlError("The user doesn't have enough rights to access this document.")

    # workspace case
    if document.user_id != str(user.id):
        # workspace is set
        if hasattr(document, 'workspace') and document.workspace is not None:
            # get list of accessible workspaces
            accessible_workspaces = workspace_api.get_all_workspaces_with_read_access_by_user(user)
            # check that accessed document belongs to an accessible workspace
            if document.workspace not in accessible_workspaces:
                raise AccessControlError("The user doesn't have enough rights to access this.")
        # workspace is not set
        else:
            raise AccessControlError("The user doesn't have enough rights to access this.")


def can_change_owner(func, document, new_user, user):
    """ Can user change document's owner.

    Args:
        func:
        document:
        new_user:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(document, new_user, user)

    if document.user_id != str(user.id):
        raise AccessControlError("The user doesn't have enough rights to access this document.")

    return func(document, new_user, user)



"""Forms for admin views
"""
from django import forms

from core_main_app.commons.validators import ExtensionValidator
from core_main_app.components.template import api as template_api
from core_main_app.components.template_version_manager.models import TemplateVersionManager
from core_main_app.components.xsl_transformation import api as xsl_transformation_api
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.views.admin.commons.upload.forms import UploadForm
from django_mongoengine.forms import DocumentForm


class EditTemplateForm(DocumentForm):
    title = forms.CharField(label='Name',
                            widget=forms.TextInput(attrs={'class': 'form-control',
                                                          'placeholder': 'Type the new name'}))

    class Meta(object):
        document = TemplateVersionManager
        fields = ['title']


class EditXSLTForm(DocumentForm):
    name = forms.CharField(label='Name',
                           widget=forms.TextInput(attrs={'class': 'form-control',
                                                         'placeholder': 'Type the new name'}))

    class Meta(object):
        document = XslTransformation
        fields = ['name']


class UploadTemplateForm(UploadForm):
    """
    Form to upload a new Template
    """

    def __init__(self, *args, **kwargs):
        super(UploadTemplateForm, self).__init__(*args, **kwargs)
        self.fields['name'].label = 'Enter Template name'
        self.fields['upload_file'].widget = forms.FileInput(attrs={'accept': '.xsd'})


class UploadVersionForm(forms.Form):
    """
    Form to upload a new version
    """
    xsd_file = forms.FileField(label='Select a file', required=True,
                               widget=forms.FileInput(attrs={'accept': '.xsd'}))


class EditProfileForm(forms.Form):
    """
    Form to edit the profile information
    """
    firstname = forms.CharField(label='First Name', max_length=100, required=True)
    lastname = forms.CharField(label='Last Name', max_length=100, required=True)
    username = forms.CharField(label='Username', max_length=100, required=True, widget=forms.HiddenInput())
    email = forms.EmailField(label='Email Address', max_length=100, required=True)


class UploadXSLTForm(UploadForm):
    """
    Form to upload a new XSLT 
    """

    def __init__(self, *args, **kwargs):
        super(UploadXSLTForm, self).__init__(*args, **kwargs)
        self.fields['name'].label = 'Enter XSLT name'
        self.fields['upload_file'].validators = [ExtensionValidator('.xsl')]
        self.fields['upload_file'].widget = forms.FileInput(attrs={'accept': '.xslt, .xsl'})


class TemplateXsltRenderingForm(forms.Form):
    """
    Form to associate list and detail XSLTs to template
    """
    id = forms.CharField(widget=forms.HiddenInput(), required=False)
    template = forms.ModelChoiceField(widget=forms.HiddenInput(), required=False, queryset=template_api.get_all())
    list_xslt = forms.ModelChoiceField(label='List XSLT', empty_label="(No XSLT)", required=False,
                                       widget=forms.Select(attrs={'class': 'form-control'}),
                                       queryset=xsl_transformation_api.get_all())
    detail_xslt = forms.ModelChoiceField(label='Detail XSLT', empty_label="(No XSLT)", required=False,
                                         widget=forms.Select(attrs={'class': 'form-control'}),
                                         queryset=xsl_transformation_api.get_all())


class TextAreaForm(forms.Form):
    """ TextArea Form
    """
    content = forms.CharField(label="", widget=forms.Textarea(attrs={'class': 'form-control'}),
                              required=False)



"""Exceptions for access control
"""


class AccessControlError(Exception):
    """ Exception raised when checking access control
    """
    def __init__(self, message):
        self.message = message



"""
Permissions API
"""
import logging

from django.contrib.auth.models import Permission, ContentType
from django.db import IntegrityError
from django.db.models import Q

from core_main_app.commons import exceptions
from core_main_app.components.group import api as group_api
from core_main_app.permissions.rights import CAN_READ_NAME, CAN_READ_CODENAME, CONTENT_TYPE_APP_LABEL, \
    CAN_WRITE_NAME, CAN_WRITE_CODENAME

logger = logging.getLogger(__name__)


def _title_to_codename(title):
    """ Change the title to a codename.

    Args:
        title:

    Returns:
    """

    # remove unnecessary spaces
    title = title.strip()
    # to lower case
    title = title.lower()
    # replace spaces by underscores
    title = title.replace(" ", "_")
    return title


def create_read_perm(title, owner_id):
    """ Create read permission.

    Args:
        title
        owner_id

    Returns:
    """
    name = CAN_READ_NAME + " - " + title.strip() + " (" + owner_id + ")"
    content_type = ContentType.objects.get(app_label=CONTENT_TYPE_APP_LABEL, model='main')
    codename = CAN_READ_CODENAME + "_" + _title_to_codename(title) + " (" + owner_id + ")"
    return _create_perm(name, content_type, codename)


def create_write_perm(title, owner_id):
    """ Create write permission.

    Args:
        title
        owner_id

    Returns:
    """
    name = CAN_WRITE_NAME + " - " + title.strip() + " (" + owner_id + ")"
    content_type = ContentType.objects.get(app_label=CONTENT_TYPE_APP_LABEL, model='main')
    codename = CAN_WRITE_CODENAME + "_" + _title_to_codename(title) + " (" + owner_id + ")"
    return _create_perm(name, content_type, codename)


def _create_perm(name, content_type, codename):
    """ Create permission.

    Args:
        name
        content_type
        codename

    Returns:
    """

    try:
        perm, created = Permission.objects.get_or_create(name=name, content_type=content_type, codename=codename)
    except IntegrityError:
        raise exceptions.NotUniqueError("The permission already exists.")
    except Exception as ie:
        raise exceptions.ModelError("Problem while creating the permission.")

    if not created:
        raise exceptions.NotUniqueError("The permission already exists.")
    return perm


def add_permission_to_user(user, permission):
    """ Add permission to user.

    Args:
        user_id
        permission

    Returns:
    """
    user.user_permissions.add(permission)
    user.save()


def add_permission_to_group(group, permission):
    """ Add permission to group.

    Args:
        group
        permission

    Returns:
    """
    group.permissions.add(permission)
    group.save()


def remove_permission_to_user(user, permission):
    """ Remove permission from user.

    Args:
        user
        permission

    Returns:
    """
    user.user_permissions.remove(permission)
    user.save()


def remove_permission_to_group(group, permission):
    """ Remove permission from group.

    Args:
        group
        permission

    Returns:
    """
    group.permissions.remove(permission)
    group.save()


def get_all_workspace_permissions_user_can_write(user):
    """ Get a list of permission ids of workspaces that the user has write access.

    Args:
        user

    Return:

    """
    # TODO: fix the super user case
    if user.is_superuser:
        return [str(perm.id) for perm in Permission.objects.filter(content_type__app_label=CONTENT_TYPE_APP_LABEL,
                                                                   codename__startswith=CAN_WRITE_CODENAME)]
    elif user.is_anonymous:
        # No permissions.
        return []
    else:
        return [str(perm.id) for perm in Permission.objects.filter((Q(user=user) | Q(group__in=user.groups.all())),
                                                                   content_type__app_label=CONTENT_TYPE_APP_LABEL,
                                                                   codename__startswith=CAN_WRITE_CODENAME)]


def get_all_workspace_permissions_user_can_read(user):
    """ Get a list of permission ids of workspaces that the user has read access.

    Args:
        user

    Return:
    """
    #TODO: fix the super user case
    if user.is_superuser:
        return [str(perm.id) for perm in Permission.objects.filter(content_type__app_label=CONTENT_TYPE_APP_LABEL,
                                                                   codename__startswith=CAN_READ_CODENAME)]
    elif user.is_anonymous:
        return [str(perm.id) for perm in Permission.objects.filter(group=group_api.get_anonymous_group(),
                                                                   content_type__app_label=CONTENT_TYPE_APP_LABEL,
                                                                   codename__startswith=CAN_READ_CODENAME)]
    else:
        return [str(perm.id) for perm in Permission.objects.filter((Q(user=user)
                                                                    | Q(group__in=user.groups.all())),
                                                                   content_type__app_label=CONTENT_TYPE_APP_LABEL,
                                                                   codename__startswith=CAN_READ_CODENAME)]


def get_by_id(permission_id):
    """ Get the permission by id.

    Args:
         permission_id:
    Returns:
    """
    return Permission.objects.get(pk=permission_id)


def delete_permission(permission_id):
    """ Delete a permission.

    Args:
        permission_id:

    Return:
    """

    try:
        perm = get_by_id(permission_id)
        perm.delete()
    except Exception as e:
        logger.warning("delete_permission threw an exception: ".format(str(e)))


def get_permission_label(permission_id):
    """ Get the label of a permission.

    Args:
        permission_id:

    Return:
    """
    permission = Permission.objects.get(pk=permission_id)
    return permission.content_type.app_label + "." + permission.codename


def check_if_group_has_perm(group, permission):
    """ Check if group has permission.

    Args:
        group:
        permission:
    Returns:
    """
    return len(group.permissions.filter(id=str(permission.id))) == 1


def get_by_codename(codename):
    """ Get the permission by codename.

    Args:
         codename:
    Returns:
    """
    return Permission.objects.get(codename=codename)



"""Url router for the REST API
"""
from django.urls import re_path
from rest_framework.urlpatterns import format_suffix_patterns

import core_main_app.rest.web_page.views as web_page_views
import core_main_app.rest.xsl_transformation.views as xsl_transformation_views
from core_main_app.rest.blob import views as blob_views
from core_main_app.rest.data import views as data_views
from core_main_app.rest.user import views as user_views
from core_main_app.rest.template import views as template_views
from core_main_app.rest.template_version_manager import views as template_version_manager_views
from core_main_app.rest.template_xsl_rendering import views as template_xsl_rendering_views
from core_main_app.rest.workspace import views as workspace_views

urlpatterns = [
    re_path(r'^template-version-manager/global/$',
            template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
            name='core_main_app_rest_template_version_manager_global_list'),

    re_path(r'^template-version-manager/user/$',
            template_version_manager_views.UserTemplateVersionManagerList.as_view(),
            name='core_main_app_rest_template_version_manager_user_list'),

    re_path(r'^template-version-manager/(?P<pk>\w+)/$',
            template_version_manager_views.TemplateVersionManagerDetail.as_view(),
            name='core_main_app_rest_template_version_manager_detail'),

    re_path(r'^template-version-manager/(?P<pk>\w+)/version/$',
            template_version_manager_views.TemplateVersion.as_view(),
            name='core_main_app_rest_template_version'),

    re_path(r'^template-version-manager/(?P<pk>\w+)/disable/$',
            template_version_manager_views.DisableTemplateVersionManager.as_view(),
            name='core_main_app_rest_template_version_manager_disable'),

    re_path(r'^template-version-manager/(?P<pk>\w+)/restore/$',
            template_version_manager_views.RestoreTemplateVersionManager.as_view(),
            name='core_main_app_rest_template_version_manager_restore'),

    re_path(r'^template/version/(?P<pk>\w+)/current/$',
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            name='core_main_app_rest_template_version_current'),

    re_path(r'^template/version/(?P<pk>\w+)/disable/$',
            template_version_manager_views.DisableTemplateVersion.as_view(),
            name='core_main_app_rest_template_version_disable'),

    re_path(r'^template/version/(?P<pk>\w+)/restore/$',
            template_version_manager_views.RestoreTemplateVersion.as_view(),
            name='core_main_app_rest_template_version_restore'),

    re_path(r'^template/xsl_rendering/$',
            template_xsl_rendering_views.TemplateXslRenderingList.as_view(),
            name='core_main_app_rest_template_detail'),

    re_path(r'^template/xsl_rendering/(?P<pk>\w+)/$',
            template_xsl_rendering_views.TemplateXslRenderingDetail.as_view(),
            name='core_main_app_rest_template_download'),

    re_path(r'^template/global/$',
            template_version_manager_views.GlobalTemplateList.as_view(),
            name='core_main_app_rest_global_template_list'),

    re_path(r'^template/user/$',
            template_version_manager_views.UserTemplateList.as_view(),
            name='core_main_app_rest_user_template_list'),

    re_path(r'^template/(?P<pk>\w+)/download/$', template_views.TemplateDownload.as_view(),
            name='core_main_app_rest_template_download'),

    re_path(r'^template/(?P<pk>\w+)/$', template_views.TemplateDetail.as_view(),
            name='core_main_app_rest_template_detail'),

    re_path(r'^data/$', data_views.DataList.as_view(),
            name='core_main_app_rest_data_list'),

    re_path(r'^data/permissions/$',
            data_views.DataPermissions.as_view(),
            name='core_main_app_rest_data_permissions'),

    re_path(r'^data/download/(?P<pk>\w+)/$', data_views.DataDownload.as_view(),
            name='core_main_app_rest_data_download'),

    re_path(r'^data/get-full$', data_views.get_by_id_with_template_info,
            name='core_main_app_rest_data_get_by_id_with_template_info'),

    re_path(r'^data/query/keyword/$', data_views.ExecuteLocalKeywordQueryView.as_view(),
            name='core_explore_common_local_query_keyword'),

    re_path(r'^data/query/$', data_views.ExecuteLocalQueryView.as_view(),
            name='core_explore_common_local_query'),

    re_path(r'^data/(?P<pk>\w+)/assign/(?P<workspace_id>\w+)$', data_views.DataAssign.as_view(),
            name='core_main_app_rest_data_assign'),

    re_path(r'^data/(?P<pk>\w+)/change-owner/(?P<user_id>\w+)$', data_views.DataChangeOwner.as_view(),
            name='core_main_app_rest_data_change_owner'),

    re_path(r'^data/(?P<pk>\w+)/$', data_views.DataDetail.as_view(),
            name='core_main_app_rest_data_detail'),

    re_path(r'^admin/blob/$', blob_views.BlobListAdmin.as_view(),
            name='core_main_app_rest_blob_list_admin'),

    re_path(r'^blob/$', blob_views.BlobList.as_view(),
            name='core_main_app_rest_blob_list'),

    re_path(r'^blobs/delete/$', blob_views.BlobDeleteList.as_view(),
            name='core_main_app_rest_blob_delete_list'),

    re_path(r'^blob/(?P<pk>\w+)/$', blob_views.BlobDetail.as_view(),
            name='core_main_app_rest_blob_detail'),

    re_path(r'^blob/download/(?P<pk>\w+)/$', blob_views.BlobDownload.as_view(),
            name='core_main_app_rest_blob_download'),

    re_path(r'^blob/(?P<pk>\w+)/assign/(?P<workspace_id>\w+)$', blob_views.BlobAssign.as_view(),
            name='core_main_app_rest_blob_assign'),

    re_path(r'^xslt/$', xsl_transformation_views.XslTransformationList.as_view(),
            name='core_main_app_rest_xslt'),

    re_path(r'^xslt/transform/$', xsl_transformation_views.XslTransformationTransform.as_view(),
            name='core_main_app_rest_xslt_transform'),

    re_path(r'^xslt/(?P<pk>\w+)/$', xsl_transformation_views.XslTransformationDetail.as_view(),
            name='core_main_app_rest_xslt_detail'),

    re_path(r'^workspace/$', workspace_views.WorkspaceList.as_view(),
            name='core_main_app_rest_workspace_list'),

    re_path(r'^workspace/(?P<pk>\w+)/detail/$', workspace_views.WorkspaceDetail.as_view(),
            name='core_main_app_rest_workspace_detail'),

    re_path(r'^workspace/read_access', workspace_views.get_workspaces_with_read_access,
            name='core_main_app_rest_workspace_get_all_workspaces_with_read_access_by_user'),

    re_path(r'^workspace/write_access', workspace_views.get_workspaces_with_write_access,
            name='core_main_app_rest_workspace_get_all_workspaces_with_write_access_by_user'),

    re_path(r'^workspace/(?P<pk>\w+)/is_public/$', workspace_views.is_workspace_public,
            name='core_main_app_rest_workspace_is_public'),

    re_path(r'^workspace/(?P<pk>\w+)/set_public/$', workspace_views.set_workspace_public,
            name='core_main_app_rest_workspace_set_public'),

    re_path(r'^workspace/(?P<pk>\w+)/set_private/$', workspace_views.set_workspace_private,
            name='core_main_app_rest_workspace_set_private'),

    re_path(r'^workspace/(?P<pk>\w+)/list_user_can_read/$',
            workspace_views.get_list_user_can_read_workspace,
            name='core_main_app_rest_workspace_list_user_can_read'),

    re_path(r'^workspace/(?P<pk>\w+)/list_user_can_write/$',
            workspace_views.get_list_user_can_write_workspace,
            name='core_main_app_rest_workspace_list_user_can_write'),

    re_path(r'^workspace/(?P<pk>\w+)/list_user_can_access/$',
            workspace_views.get_list_user_can_access_workspace,
            name='core_main_app_rest_workspace_list_user_can_access'),

    re_path(r'^workspace/(?P<pk>\w+)/list_group_can_read/$',
            workspace_views.get_list_group_can_read_workspace,
            name='core_main_app_rest_workspace_list_group_can_read'),

    re_path(r'^workspace/(?P<pk>\w+)/list_group_can_write/$',
            workspace_views.get_list_group_can_write_workspace,
            name='core_main_app_rest_workspace_list_group_can_write'),

    re_path(r'^workspace/(?P<pk>\w+)/list_group_can_access/$',
            workspace_views.get_list_group_can_access_workspace,
            name='core_main_app_rest_workspace_list_group_can_access'),

    re_path(r'^workspace/(?P<pk>\w+)/add_read_right_to_user/(?P<user_id>\w+)/$',
            workspace_views.add_user_read_right_to_workspace,
            name='core_main_app_rest_workspace_add_user_read'),

    re_path(r'^workspace/(?P<pk>\w+)/add_write_right_to_user/(?P<user_id>\w+)/$',
            workspace_views.add_user_write_right_to_workspace,
            name='core_main_app_rest_workspace_add_user_write'),

    re_path(r'^workspace/(?P<pk>\w+)/add_read_right_to_group/(?P<group_id>\w+)/$',
            workspace_views.add_group_read_right_to_workspace,
            name='core_main_app_rest_workspace_add_group_read'),

    re_path(r'^workspace/(?P<pk>\w+)/add_write_right_to_group/(?P<group_id>\w+)/$',
            workspace_views.add_group_write_right_to_workspace,
            name='core_main_app_rest_workspace_add_group_write'),

    re_path(r'^workspace/(?P<pk>\w+)/remove_read_right_to_user/(?P<user_id>\w+)/$',
            workspace_views.remove_user_read_right_to_workspace,
            name='core_main_app_rest_workspace_remove_user_read'),

    re_path(r'^workspace/(?P<pk>\w+)/remove_write_right_to_user/(?P<user_id>\w+)/$',
            workspace_views.remove_user_write_right_to_workspace,
            name='core_main_app_rest_workspace_remove_user_write'),

    re_path(r'^workspace/(?P<pk>\w+)/remove_read_right_to_group/(?P<group_id>\w+)/$',
            workspace_views.remove_group_read_right_to_workspace,
            name='core_main_app_rest_workspace_remove_group_read'),

    re_path(r'^workspace/(?P<pk>\w+)/remove_write_right_to_group/(?P<group_id>\w+)/$',
            workspace_views.remove_group_write_right_to_workspace,
            name='core_main_app_rest_workspace_remove_group_write'),

    re_path(r'^user/(?P<pk>[\w-]+)/$', user_views.UserDetail.as_view(),
            name='core_main_app_user_detail'),

    re_path(r'^workspace/(?P<workspace_id>\w+)/data/$',
            data_views.DataListByWorkspace.as_view(),
            name='core_main_app_rest_data_workspace_list'),

    re_path(r'^blob/(?P<pk>\w+)/change-owner/(?P<user_id>\w+)$', blob_views.BlobChangeOwner.as_view(),
            name='core_main_app_rest_blob_change_owner'),

    re_path(r'^user/$', user_views.UserList.as_view(),
            name='core_main_app_user_detail'),

    re_path(r'^login-page$',
            web_page_views.WebPageList.as_view(
                web_page_type="login"
            ),
            name='login_page_rest_views')
]

urlpatterns = format_suffix_patterns(urlpatterns)



from os import chdir, pardir
from os.path import join, exists, dirname, normpath, abspath
from re import sub

from setuptools import find_packages, setup


def req_link(external_url):
    egg_link = sub(r'https://[^=]+=', '', external_url)
    return "==".join(egg_link.rsplit("-", 1))


reqs_default = join(dirname(__file__), "requirements.txt")
reqs_core = join(dirname(__file__), "requirements.core.txt")
required = []

if exists(reqs_default):
    with open(reqs_default) as f:
        required += f.read().splitlines()

if exists(reqs_core):
    with open(reqs_core) as f:
        required += f.read().splitlines()

dep_links = [r for r in required if r.startswith("https://")]
required = [req_link(r) if r.startswith("https://") else r for r in required]

with open(join(dirname(__file__), "README.rst")) as f:
    long_desc = f.read()

# Allow setup.py to be run from any path
chdir(normpath(join(abspath(__file__), pardir)))

setup(
    name="core_main_app",
    version="1.9.1",
    description="Main functionalities for the curator core project",
    long_description=long_desc,
    author="NIST IT Lab",
    author_email="itl_inquiries@nist.gov",
    url="https://github.com/usnistgov/core_main_app",
    packages=find_packages(),
    include_package_data=True,
    install_requires=required,
    dependency_links=dep_links,
)



""" Mock a Request object.
"""
import json

from django.core.wsgi import get_wsgi_application
from django.http import HttpResponse
from rest_framework import status
from rest_framework.test import APIRequestFactory


class RequestMock(object):
    """ Represent a request.
        Use this class to simulate an HTTP request.
    """

    @staticmethod
    def do_request_get(view, user, data=None, param=None):
        """Execute a GET HTTP request.
        Args:
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.

        Returns:
            Response: Request response.

        """
        return RequestMock._do_request("GET", view, user, data, param)

    @staticmethod
    def do_request_post(view, user, data=None, param=None):
        """Execute a POST HTTP request.
        Args:
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.

        Returns:
            Response: Request response.
        """
        return RequestMock._do_request("POST", view, user, data, param)

    @staticmethod
    def do_request_put(view, user, data=None, param=None):
        """Execute a PUT HTTP request.
        Args:
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.

        Returns:
            Response: Request response.

        """
        return RequestMock._do_request("PUT", view, user, data, param)

    @staticmethod
    def do_request_delete(view, user, data=None, param=None):
        """Execute a DELETE HTTP request.
        Args:
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.

        Returns:
            Response: Request response.

        """
        return RequestMock._do_request("DELETE", view, user, data, param)

    @staticmethod
    def do_request_patch(view, user, data=None, param=None):
        """Execute a PATCH HTTP request.
        Args:
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.

        Returns:
            Response: Request response.

        """
        return RequestMock._do_request("PATCH", view, user, data, param)

    @staticmethod
    def _do_request(http_method, view, user, data=None, param=None):
        """Execute the http_method request.
        Args:
            http_method: HTTP method.
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.

        Returns:
            Response: Request response.

        """
        url = "/dummy_url"
        factory = APIRequestFactory()
        # Request by http_method.
        if http_method == "GET":
            request = factory.get(url, data=data)
        elif http_method == "POST":
            request = factory.post(url, data=json.dumps(data), content_type="application/json")
        elif http_method == "PUT":
            request = factory.put(url, data=json.dumps(data), content_type="application/json")
        elif http_method == "DELETE":
            request = factory.delete(url, data=json.dumps(data), content_type="application/json")
        elif http_method == "PATCH":
            request = factory.patch(url, data=json.dumps(data), content_type="application/json")
        else:
            return HttpResponse(status=status.HTTP_405_METHOD_NOT_ALLOWED)
        # Set the user
        request.user = user

        # i18n. Get django validation messages.
        get_wsgi_application()
        # Do not use CSRF checks.
        request._dont_enforce_csrf_checks = True

        if param:
            view_ = view(request, **param)
        else:
            view_ = view(request)

        return view_



""" Authentication tests for Template Version Manager REST API
"""
from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status

from core_main_app.components.template.models import Template
from core_main_app.components.template_version_manager.models import TemplateVersionManager
from core_main_app.components.version_manager.models import VersionManager
from core_main_app.rest.template_version_manager import views as template_version_manager_views
from core_main_app.rest.template_version_manager.serializers import TemplateVersionManagerSerializer, \
    CreateTemplateSerializer
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestGlobalTemplateVersionManagerListGetPermission(SimpleTestCase):

    @patch.object(TemplateVersionManager, "get_global_version_managers")
    def test_anonymous_returns_http_200(self, template_version_manager_get_all_global_version_managers):
        template_version_manager_get_all_global_version_managers.return_value = {}

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(TemplateVersionManager, "get_global_version_managers")
    def test_authenticated_returns_http_200(self, template_version_manager_get_all_global_version_managers):
        template_version_manager_get_all_global_version_managers.return_value = {}

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(TemplateVersionManager, "get_global_version_managers")
    def test_staff_returns_http_200(self, template_version_manager_get_all_global_version_managers):
        template_version_manager_get_all_global_version_managers.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestUserTemplateVersionManagerListGetPermission(SimpleTestCase):

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerList.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(TemplateVersionManager, "get_all_version_manager_by_user_id")
    def test_authenticated_returns_http_200(self, template_version_manager_get_all_version_manager_by_user_id):
        template_version_manager_get_all_version_manager_by_user_id.return_value = {}

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(TemplateVersionManager, "get_all_version_manager_by_user_id")
    def test_staff_returns_http_200(self, template_version_manager_get_all_version_manager_by_user_id):
        template_version_manager_get_all_version_manager_by_user_id.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateVersionManagerDetailGetPermission(SimpleTestCase):

    def setUp(self):
        self.fake_id = '507f1f77bcf86cd799439011'

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            template_version_manager_views.TemplateVersionManagerDetail.as_view(),
            None,
            param={'pk': self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(VersionManager, "get_by_id")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_authenticated_returns_http_200(self, template_version_manager_data, version_manager_get_by_id):
        template_version_manager_data.return_value = True
        version_manager_get_by_id.return_value = {}

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            template_version_manager_views.TemplateVersionManagerDetail.as_view(),
            mock_user,
            param={'pk': self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(VersionManager, "get_by_id")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_staff_returns_http_200(self, template_version_manager_data, version_manager_get_by_id):
        template_version_manager_data.return_value = True
        version_manager_get_by_id.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.TemplateVersionManagerDetail.as_view(),
            mock_user,
            param={'pk': self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateVersionPostPermission(SimpleTestCase):

    def setUp(self):
        self.fake_id = '507f1f77bcf86cd799439011'

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(
            template_version_manager_views.TemplateVersion.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_post(
            template_version_manager_views.TemplateVersion.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(VersionManager, "get_by_id")
    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    def test_staff_returns_http_201(self, template_version_manager_serializer_data,
                                    template_version_manager_serializer_save,
                                    template_version_manager_serializer_valid,
                                    version_manager_get_by_id):
        version_manager_get_by_id.return_value = {}
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_post(
            template_version_manager_views.TemplateVersion.as_view(),
            mock_user,
            data={},
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestUserTemplateListPostPermission(SimpleTestCase):

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(
            template_version_manager_views.UserTemplateList.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    @patch.object(TemplateVersionManagerSerializer, "is_valid")
    @patch.object(TemplateVersionManagerSerializer, "save")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_authenticated_returns_http_201(self, template_version_manager_serializer_data,
                                            template_version_manager_serializer_save,
                                            template_version_manager_serializer_valid,
                                            create_template_version_manager_serializer_data,
                                            create_template_version_manager_serializer_save,
                                            create_template_version_manager_serializer_valid):
        create_template_version_manager_serializer_data.return_value = True
        create_template_version_manager_serializer_save.return_value = None
        create_template_version_manager_serializer_valid.return_value = {}
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_post(
            template_version_manager_views.UserTemplateList.as_view(),
            mock_user,
            data={}
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    @patch.object(TemplateVersionManagerSerializer, "is_valid")
    @patch.object(TemplateVersionManagerSerializer, "save")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_staff_returns_http_201(self, template_version_manager_serializer_data,
                                    template_version_manager_serializer_save,
                                    template_version_manager_serializer_valid,
                                    create_template_version_manager_serializer_data,
                                    create_template_version_manager_serializer_save,
                                    create_template_version_manager_serializer_valid):
        create_template_version_manager_serializer_data.return_value = True
        create_template_version_manager_serializer_save.return_value = None
        create_template_version_manager_serializer_valid.return_value = {}
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_post(
            template_version_manager_views.UserTemplateList.as_view(),
            mock_user,
            data={}
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestGlobalTemplateListPostPermission(SimpleTestCase):

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(
            template_version_manager_views.GlobalTemplateList.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_post(
            template_version_manager_views.GlobalTemplateList.as_view(),
            mock_user,
            data={}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    @patch.object(TemplateVersionManagerSerializer, "is_valid")
    @patch.object(TemplateVersionManagerSerializer, "save")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_staff_returns_http_201(self, template_version_manager_serializer_data,
                                    template_version_manager_serializer_save,
                                    template_version_manager_serializer_valid,
                                    create_template_version_manager_serializer_data,
                                    create_template_version_manager_serializer_save,
                                    create_template_version_manager_serializer_valid):
        create_template_version_manager_serializer_data.return_value = True
        create_template_version_manager_serializer_save.return_value = None
        create_template_version_manager_serializer_valid.return_value = {}
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_post(
            template_version_manager_views.GlobalTemplateList.as_view(),
            mock_user,
            data={}
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestCurrentTemplateVersionPatchPermission(SimpleTestCase):

    def setUp(self):
        self.fake_id = '507f1f77bcf86cd799439011'

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            mock_user,
            data={}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch('core_main_app.components.version_manager.api.set_current')
    @patch('core_main_app.components.version_manager.api.get_from_version')
    @patch.object(Template, "get_by_id")
    def test_staff_returns_http_200(self, template_get_by_id, version_manager_get_from_version,
                                    version_manager_set_current):
        version_manager_set_current.return_value = {}
        version_manager_get_from_version.return_value = {}
        template_get_by_id.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            mock_user,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDisableTemplateVersionPatchPermission(SimpleTestCase):

    def setUp(self):
        self.fake_id = '507f1f77bcf86cd799439011'

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersion.as_view(),
            None,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersion.as_view(),
            mock_user,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch('core_main_app.components.version_manager.api.disable_version')
    @patch('core_main_app.components.version_manager.api.get_from_version')
    @patch.object(Template, "get_by_id")
    def test_staff_returns_http_200(self, template_get_by_id, version_manager_get_from_version,
                                    version_manager_disable_version):
        version_manager_disable_version.return_value = {}
        version_manager_get_from_version.return_value = {}
        template_get_by_id.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersion.as_view(),
            mock_user,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRestoreTemplateVersionPatchPermission(SimpleTestCase):

    def setUp(self):
        self.fake_id = '507f1f77bcf86cd799439011'

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersion.as_view(),
            None,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersion.as_view(),
            mock_user,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch('core_main_app.components.version_manager.api.restore_version')
    @patch('core_main_app.components.version_manager.api.get_from_version')
    @patch.object(Template, "get_by_id")
    def test_staff_returns_http_200(self, template_get_by_id, version_manager_get_from_version,
                                    version_manager_restore_version):
        version_manager_restore_version.return_value = {}
        version_manager_get_from_version.return_value = {}
        template_get_by_id.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersion.as_view(),
            mock_user,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDisableTemplateVersionManagerPatchPermission(SimpleTestCase):

    def setUp(self):
        self.fake_id = '507f1f77bcf86cd799439011'

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersionManager.as_view(),
            None,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(VersionManager, "get_by_id")
    @patch('core_main_app.components.version_manager.api.disable')
    def test_authenticated_returns_http_200(self, version_manager_disable, version_manager_get_by_id):
        version_manager_get_by_id.return_value = TemplateVersionManager(user='1')
        version_manager_disable.return_value = {}
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersionManager.as_view(),
            mock_user,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(VersionManager, "get_by_id")
    @patch('core_main_app.components.version_manager.api.disable')
    def test_staff_returns_http_200(self, version_manager_disable, version_manager_get_by_id):
        version_manager_get_by_id.return_value = TemplateVersionManager(user='2')
        version_manager_disable.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersionManager.as_view(),
            mock_user,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRestoreTemplateVersionManagerPatchPermission(SimpleTestCase):

    def setUp(self):
        self.fake_id = '507f1f77bcf86cd799439011'

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersionManager.as_view(),
            None,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch('core_main_app.components.version_manager.api.restore')
    @patch.object(VersionManager, "get_by_id")
    def test_authenticated_returns_http_200(self, version_manager_get_by_id, version_manager_restore):
        version_manager_restore.return_value = {}
        version_manager_get_by_id.return_value = TemplateVersionManager(user='1')
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersionManager.as_view(),
            mock_user,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch('core_main_app.components.version_manager.api.restore')
    @patch.object(VersionManager, "get_by_id")
    def test_staff_returns_http_200(self, version_manager_get_by_id, version_manager_restore):
        version_manager_restore.return_value = {}
        version_manager_get_by_id.return_value = TemplateVersionManager(user='2')

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersionManager.as_view(),
            mock_user,
            param={'pk': self.fake_id}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



""" Settings for core_main_app. These settings are overwritten at project level.
"""
import os
from os.path import join

from django.conf import settings

if not settings.configured:
    settings.configure()

CUSTOM_NAME = getattr(settings, 'CUSTOM_NAME', 'Local')
""" :py:class:`str`: Name of the local instance
"""

SECRET_KEY = getattr(settings, 'SECRET_KEY', '<secret_key>')
""" :py:class:`str`: Django application secret key.
"""

SERVER_URI = getattr(settings, 'SERVER_URI', "http://127.0.0.1:8000")
""" :py:class:`str`: Server URI for import reference.
"""

INSTALLED_APPS = getattr(settings, 'INSTALLED_APPS', [])
""" :py:class:`list`: List of apps installed.
"""

# Website configuration
WEBSITE_COMPACT_TITLE = getattr(settings, 'WEBSITE_COMPACT_TITLE', "Project")
""" :py:class:`str`: Website title.
"""

# Choose from:  black, black-light, blue, blue-light, green, green-light, purple, purple-light, red, red-light, yellow,
#               yellow-light
WEBSITE_ADMIN_COLOR = getattr(settings, 'WEBSITE_ADMIN_COLOR', "yellow")
""" :py:class:`str`: color of the admin dashboard.
"""

# XML
XERCES_VALIDATION = getattr(settings, 'XERCES_VALIDATION', False)
""" :py:class:`bool`: Enables Xerces validation (requires additional packages).
"""

# GridFS
GRIDFS_DATA_COLLECTION = getattr(settings, 'GRIDFS_DATA_COLLECTION', 'fs_data')
""" :py:class:`str`: Collection name for file storage in MongoDB.
"""

# Celery configuration
USE_BACKGROUND_TASK = getattr(settings, 'USE_BACKGROUND_TASK', False)
""" :py:class:`bool`: Define use of celery for background tasks.
"""

BROKER_URL = getattr(settings, 'BROKER_URL', 'redis://localhost:6379/0')
""" :py:class:`str`: Celery broker address.
"""

broker_transport_default = {
    "visibility_timeout": 3600,
    'fanout_prefix': True,
    'fanout_patterns': True
}
BROKER_TRANSPORT_OPTIONS = getattr(settings, 'BROKER_TRANSPORT_OPTIONS', broker_transport_default)
""" :py:class:`dict`: Celery broker options.
"""

CELERY_RESULT_BACKEND = getattr(settings, 'CELERY_RESULT_BACKEND', 'redis://localhost:6379/0')
""" :py:class:`str`: Celery backend for results.
"""

# SMTP Configuration
USE_EMAIL = getattr(settings, 'USE_EMAIL', False)
""" :py:class:`bool`: Activate email sending on the platform.
"""

SERVER_EMAIL = getattr(settings, 'SERVER_EMAIL', 'noreply@example.com')
""" :py:class:`str`: Email address sending the message.
"""

ADMINS = getattr(settings, 'ADMINS', [('admin', 'admin@example.com')])
""" :py:class:`list`: Email addresses of admins.
"""

MANAGERS = getattr(settings, 'MANAGERS', [('manager', 'moderator@example.com')])
""" :py:class:`list`: Email addresses of moderators (managers). 
"""

EMAIL_SUBJECT_PREFIX = getattr(settings, 'EMAIL_SUBJECT_PREFIX', "[CURATOR] ")
""" :py:class:`str`: Prefix for easy classification of emails.
"""

MONGO_USER = getattr(settings, 'MONGO_USER', "mgi_user")
""" :py:class:`str`: MongoDB user.
"""

MONGO_PASSWORD = getattr(settings, 'MONGO_PASSWORD', "mgi_password")
""" :py:class:`str`: MongoDB password.
"""

DB_NAME = getattr(settings, 'DB_NAME', "mgi")
""" :py:class:`str`: MongoDB password.
"""

mongodb_uri_default = "mongodb://" + MONGO_USER + ":" + MONGO_PASSWORD + "@localhost/" + DB_NAME
MONGODB_URI = getattr(settings, 'MONGODB_URI', mongodb_uri_default)
""" :py:class:`str`: MongoDB connection URI. Automatically generated from other parameters.
"""

USE_TZ = getattr(settings, 'USE_TZ', True)
""" :py:class:`bool`: Activate use of timezones.
"""

SEARCHABLE_DATA_OCCURRENCES_LIMIT = getattr(settings, 'SEARCHABLE_DATA_OCCURRENCES_LIMIT', None)
""" :py:class:`int` | :py:attr:`None`: Limit for number of occurent to be returned by a search.
"""

BLOB_HOST = getattr(settings, 'BLOB_HOST', 'GRIDFS')
""" :py:class:`str`: Type of file storage.
"""

BLOB_HOST_URI = getattr(settings, 'BLOB_HOST_URI', MONGODB_URI)
""" :py:class:`str`: File storage system URI.
"""

BLOB_HOST_USER = getattr(settings, 'BLOB_HOST_USER', None)
""" :py:class:`str`: User for file storage.
"""

BLOB_HOST_PASSWORD = getattr(settings, 'BLOB_HOST_PASSWORD', None)
""" :py:class:`str`: Password for file storage.
"""

# Password settings for django.contrib.auth validators
PASSWORD_MIN_LENGTH = getattr(settings, 'PASSWORD_MIN_LENGTH', 0)
""" :py:class:`int`: Required minimum length of a password.
"""

PASSWORD_MIN_LOWERCASE_LETTERS = getattr(settings, 'PASSWORD_MIN_LOWERCASE_LETTERS', 0)
""" :py:class:`int`: Required number of lowercase chars in a password.
"""

PASSWORD_MIN_UPPERCASE_LETTERS = getattr(settings, 'PASSWORD_MIN_UPPERCASE_LETTERS', 0)
""" :py:class:`int`: Required number of uppercase chars in a password.
"""

PASSWORD_MIN_LETTERS = getattr(settings, 'PASSWORD_MIN_LETTERS', 0)
""" :py:class:`int`: Specifies the minimum amount of required letters in a password.
"""

PASSWORD_MIN_NUMBERS = getattr(settings, 'PASSWORD_MIN_NUMBERS', 0)
""" :py:class:`int`: Specifies the minimum amount of required numbers in a password.
"""

PASSWORD_MIN_SYMBOLS = getattr(settings, 'PASSWORD_MIN_SYMBOLS', 0)
""" :py:class:`int`: Specifies the minimum amount of required symbols in a password.
"""

PASSWORD_MAX_OCCURRENCE = getattr(settings, 'PASSWORD_MAX_OCCURRENCE', None)
""" :py:class:`int`: Specifies the maximum amount of consecutive characters allowed in passwords.
"""

# Lock
LOCK_OBJECT_TTL = getattr(settings, 'LOCK_OBJECT_TTL', 600)  # 10 min
""" :py:class:`int`: Lock duration on files.
"""

# Results per page for paginator
RESULTS_PER_PAGE = getattr(settings, 'RESULTS_PER_PAGE', 10)
""" :py:class:`int`: Results per page.
"""

CAN_SET_PUBLIC_DATA_TO_PRIVATE = getattr(settings, 'CAN_SET_PUBLIC_DATA_TO_PRIVATE', True)
""" :py:class:`bool`: Can set public data to private.
"""

DEFAULT_DATA_RENDERING_XSLT = getattr(settings, 'DEFAULT_DATA_RENDERING_XSLT',
                                      join('core_main_app', 'common', 'xsl', 'xml2html.xsl'))
""" :py:class:`str`: Path to default XSLT to render data.
"""

# Can anonymous access public document
CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT = getattr(settings, 'CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT', False)
""" :py:class:`bool`: Can anonymous user access public document.
"""

DISPLAY_NIST_HEADERS = getattr(settings, 'DISPLAY_NIST_HEADERS', False)
""" :py:class:`bool`: HTML pages show the NIST headers/footers.
"""

# Locale folder
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
""" :py:class:`str`: Project installation directory.
"""

LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'core_main_app/locale'),
)
""" :py:class:`list`: Path for translation messages.
"""

CAN_SET_WORKSPACE_PUBLIC = getattr(settings, 'CAN_SET_WORKSPACE_PUBLIC', True)
""" :py:class:`bool`: Can set workspace public.
"""

SSL_CERTIFICATES_DIR = getattr(settings, 'SSL_CERTIFICATES_DIR', True)
""" :py:class:`str`: SSL certificates directory location.
"""

XSD_URI_RESOLVER = getattr(settings, 'XSD_URI_RESOLVER', None)
""" :py:class:`str`: XSD URI Resolver for lxml validation. Choose from:  None, 'REQUESTS_RESOLVER'.
"""

VERIFY_DATA_ACCESS = getattr(settings, 'VERIFY_DATA_ACCESS', False)
""" :py:class:`bool`: Verify that data returned by a query can be accessed.
"""

DATA_SORTING_FIELDS = getattr(settings, 'DATA_SORTING_FIELDS', [])
""" ::py:class:`str` Set the default sort fields for the data query. all the field must be prefixed by "+" or "-" (asc or desc sort)
    the sort can be multi field and each field must be delimited by "," (ex. ["-title","+name","+date"]) 
"""



"""
Encode the date correctly for Django password
"""

from datetime import datetime
from json import JSONEncoder


class DateTimeEncoder(JSONEncoder):
    """ Instead of letting the default encoder convert datetime to string,
        convert datetime objects into a dict, which can be decoded by the
        DateTimeDecoder
    """

    def default(self, obj):
        """

        Args:
            obj:

        Returns:

        """
        if isinstance(obj, datetime):
            return {
                '__type__': 'datetime',
                'year': obj.year,
                'month': obj.month,
                'day': obj.day,
                'hour': obj.hour,
                'minute': obj.minute,
                'second': obj.second,
                'microsecond': obj.microsecond,
            }
        else:
            return JSONEncoder.default(self, obj)



"""Utils for mongodb raw query
"""


def add_access_criteria(query, accessible_workspaces, user):
    """ Add access criteria to the query.

    Args:
        query:
        accessible_workspaces:
        user:

    Returns:

    """
    access_criteria = _get_accessible_criteria(accessible_workspaces, user)
    # add access criteria to original query
    query = {'$and': [query, access_criteria]}
    # return query
    return query


def add_aggregate_access_criteria(query, accessible_workspaces, user):
    """ Add access criteria to an aggregation query.

    Args:
        query:
        accessible_workspaces:
        user:

    Returns:

    """
    has_match_criteria = False
    access_criteria = _get_accessible_criteria(accessible_workspaces, user)
    # Update $match with access criteria if exist.
    for elt in query:
        if '$match' in elt:
            elt['$match'].update(access_criteria)
            has_match_criteria = True
            break
    # Add $match with access criteria if does not exist.
    if not has_match_criteria:
        query.insert(0, {'$match': access_criteria})

    return query


def _get_accessible_criteria(accessible_workspaces, user):
    """ Get accessible criteria.

    Args:
        accessible_workspaces:
        user:

    Returns:

    """
    # workspace should be in list of accessible workspaces
    workspace_criteria = {'workspace': {"$in": accessible_workspaces}}
    # user_id should have the id of the user making the query
    user_criteria = {'user_id': str(user.id)}
    # access granted if workspace or user criteria true
    access_criteria = {'$or': [workspace_criteria, user_criteria]}
    # return access criteria
    return access_criteria



""" Unit tests of system API
"""
from collections import OrderedDict
from unittest.case import TestCase

from bson import ObjectId
from mock import patch

import core_main_app.system.api as system_api
from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.components.workspace.models import Workspace


def _get_template():
    template = Template()
    template.pk = 1
    xsd = "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>" \
          "<xs:element name='tag'></xs:element></xs:schema>"
    template.content = xsd
    return template


def _get_workspace():
    workspace = Workspace()
    workspace.title = "title"
    workspace.owner = 1
    workspace.is_public = True
    workspace.pk = ObjectId()
    return workspace


class TestSystemGetAllExcept(TestCase):

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_return_data_object(self, mock_get_all_except):
        mock_data = Data(template=_get_template(), user_id="1", dict_content=OrderedDict(),
                         title="title")
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except(["1"])
        self.assertTrue(all(isinstance(item, Data) for item in result))

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_return_correct_count(self, mock_get_all_except):
        mock_data = Data(template=_get_template(), user_id="1", dict_content=OrderedDict(),
                         title="title")
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except(["1"])
        self.assertEqual(len(result), 1)

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_empty_list_return_data_object(self, mock_get_all_except):
        mock_data = Data(template=_get_template(), user_id="1", dict_content=OrderedDict(),
                         title="title")
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except([])
        self.assertTrue(all(isinstance(item, Data) for item in result))

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_empty_list_return_correct_count(self, mock_get_all_except):
        mock_data = Data(template=_get_template(), user_id="1", dict_content=OrderedDict(),
                         title="title")
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except([])
        self.assertEqual(len(result), 1)

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_inexistant_id_return_data_object(self, mock_get_all_except):
        mock_data = Data(template=_get_template(), user_id="1", dict_content=OrderedDict(),
                         title="title")
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except(["1"])
        self.assertTrue(all(isinstance(item, Data) for item in result))

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_inexistant_id_return_correct_count(self, mock_get_all_except):
        mock_data = Data(template=_get_template(), user_id="1", dict_content=OrderedDict(),
                         title="title")
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except(["1"])
        self.assertEqual(len(result), 1)


class TestGetAllDataInWorkspaces(TestCase):
    @patch.object(Data, "get_all_by_list_workspace")
    def test_returns_data_object(self, mock_get_all_by_list_workspace):
        mock_data = Data(template=_get_template(), workspace=_get_workspace(),
                         user_id="1", dict_content=OrderedDict(),
                         title="mock_title")
        mock_get_all_by_list_workspace.return_value = [mock_data]

        result = system_api.get_all_data_in_workspaces(
            [_get_workspace().pk]
        )
        self.assertTrue(all(isinstance(item, Data) for item in result))

    @patch.object(Data, "get_all_by_list_workspace")
    def test_returns_correct_count(self, mock_get_all_by_list_workspace):
        mock_data = Data(template=_get_template(), workspace=_get_workspace(),
                         user_id="1", dict_content=OrderedDict(),
                         title="mock_title")
        mock_get_all_by_list_workspace.return_value = [mock_data]

        result = system_api.get_all_data_in_workspaces(
            [_get_workspace().pk]
        )
        self.assertEqual(len(result), 1)

    @patch.object(Data, "get_all_by_list_workspace")
    def test_none_returns_data_object(self, mock_get_all_by_list_workspace):
        mock_data = Data(template=_get_template(), workspace=_get_workspace(),
                         user_id="1", dict_content=OrderedDict(),
                         title="mock_title")
        mock_get_all_by_list_workspace.return_value = [mock_data]

        result = system_api.get_all_data_in_workspaces([None])
        self.assertTrue(all(isinstance(item, Data) for item in result))

    @patch.object(Data, "get_all_by_list_workspace")
    def test_none_returns_correct_count(self, mock_get_all_by_list_workspace):
        mock_data = Data(template=_get_template(), workspace=_get_workspace(),
                         user_id="1", dict_content=OrderedDict(),
                         title="mock_title")
        mock_get_all_by_list_workspace.return_value = [mock_data]

        result = system_api.get_all_data_in_workspaces([None])
        self.assertEqual(len(result), 1)

    @patch.object(Data, "get_all_by_list_workspace")
    def test_empty_list_returns_no_data(self, mock_get_all_by_list_workspace):
        mock_get_all_by_list_workspace.return_value = []

        result = system_api.get_all_data_in_workspaces([])
        self.assertEqual(len(result), 0)

    @patch.object(Data, "get_all_by_list_workspace")
    def test_invalid_workspace_returns_no_data(
            self, mock_get_all_by_list_workspace
    ):
        mock_get_all_by_list_workspace.return_value = []

        result = system_api.get_all_data_in_workspaces([ObjectId()])
        self.assertEqual(len(result), 0)


class TestGetAllDataInWorkspacesForTemplates(TestCase):
    @patch.object(Data, "get_all_by_templates_and_workspaces")
    def test_returns_data_object(self,
                                 mock_get_all_by_templates_and_workspaces):
        mock_data = Data(template=_get_template(), workspace=_get_workspace(),
                         user_id="1", dict_content=OrderedDict(),
                         title="mock_title")
        mock_get_all_by_templates_and_workspaces.return_value = [mock_data]

        result = system_api.get_all_data_in_workspaces_for_templates(
            [_get_workspace().pk], [_get_template().pk]
        )
        self.assertTrue(all(isinstance(item, Data) for item in result))

    @patch.object(Data, "get_all_by_templates_and_workspaces")
    def test_returns_correct_count(
            self, mock_get_all_by_templates_and_workspaces
    ):
        mock_data = Data(template=_get_template(), workspace=_get_workspace(),
                         user_id="1", dict_content=OrderedDict(),
                         title="mock_title")
        mock_get_all_by_templates_and_workspaces.return_value = [mock_data]

        result = system_api.get_all_data_in_workspaces_for_templates(
            [_get_workspace().pk], [_get_template().pk]
        )
        self.assertEqual(len(result), 1)

    @patch.object(Data, "get_all_by_templates_and_workspaces")
    def test_null_workspace_returns_data_object(
            self, mock_get_all_by_templates_and_workspaces
    ):
        mock_data = Data(template=_get_template(), workspace=_get_workspace(),
                         user_id="1", dict_content=OrderedDict(),
                         title="mock_title")
        mock_get_all_by_templates_and_workspaces.return_value = [mock_data]

        result = system_api.get_all_data_in_workspaces_for_templates(
            [None], [_get_template().pk]
        )
        self.assertTrue(all(isinstance(item, Data) for item in result))

    @patch.object(Data, "get_all_by_templates_and_workspaces")
    def test_null_workspace_returns_correct_count(
            self, mock_get_all_by_templates_and_workspaces
    ):
        mock_data = Data(template=_get_template(), workspace=_get_workspace(),
                         user_id="1", dict_content=OrderedDict(),
                         title="mock_title")
        mock_get_all_by_templates_and_workspaces.return_value = [mock_data]

        result = system_api.get_all_data_in_workspaces_for_templates(
            [None], [_get_template().pk]
        )
        self.assertEqual(len(result), 1)

    @patch.object(Data, "get_all_by_templates_and_workspaces")
    def test_empty_workspaces_returns_no_data(
            self, mock_get_all_by_templates_and_workspaces
    ):
        mock_get_all_by_templates_and_workspaces.return_value = []

        result = system_api.get_all_data_in_workspaces_for_templates(
            [], [_get_template().pk]
        )
        self.assertEqual(len(result), 0)

    @patch.object(Data, "get_all_by_templates_and_workspaces")
    def test_empty_templates_returns_no_data(
            self, mock_get_all_by_templates_and_workspaces
    ):
        mock_get_all_by_templates_and_workspaces.return_value = []

        result = system_api.get_all_data_in_workspaces_for_templates(
            [_get_workspace().pk], []
        )
        self.assertEqual(len(result), 0)

    @patch.object(Data, "get_all_by_templates_and_workspaces")
    def test_invalid_workspace_returns_no_data(
            self, mock_get_all_by_templates_and_workspaces
    ):
        mock_get_all_by_templates_and_workspaces.return_value = []

        result = system_api.get_all_data_in_workspaces_for_templates(
            [ObjectId()], [_get_template().pk]
        )
        self.assertEqual(len(result), 0)

    @patch.object(Data, "get_all_by_templates_and_workspaces")
    def test_invalid_template_returns_no_data(
            self, mock_get_all_by_templates_and_workspaces
    ):
        mock_get_all_by_templates_and_workspaces.return_value = []

        result = system_api.get_all_data_in_workspaces_for_templates(
            [_get_workspace().pk], [ObjectId()]
        )
        self.assertEqual(len(result), 0)



""" BLOB API
"""
import core_main_app.commons.exceptions as exceptions
from core_main_app.access_control.api import has_perm_administration, can_read_or_write_in_workspace, can_read, \
    can_read_id, can_write
from core_main_app.access_control.decorators import access_control
from core_main_app.components.blob.access_control import can_write_blob_workspace
from core_main_app.components.blob.models import Blob
from core_main_app.access_control.api import can_change_owner

def insert(blob):
    """ Insert the blob in the blob repository.

    Args:
        blob:

    Returns:

    """
    # if blob is not set
    if blob.blob is None:
        raise exceptions.ApiError("Unable to save the blob: blob field is not set.")
    # save blob on blob host
    blob.save_blob()
    # save blob in database
    return blob.save()


@access_control(can_write_blob_workspace)
def assign(blob, workspace, user):
    """ Assign blob to a workspace.

    Args:
        blob:
        workspace:
        user:

    Returns:

    """
    blob.workspace = workspace
    return blob.save()


@access_control(can_write)
def delete(blob, user):
    """ Delete the blob.

    Args:
        blob:

    Returns:

    """
    # delete blob on blob host
    blob.delete_blob()
    # delete blob in database
    return blob.delete()


@access_control(can_read_id)
def get_by_id(blob_id, user):
    """ Return blob by its id.

    Args:
        blob_id:

    Returns:

    """
    return Blob.get_by_id(blob_id)


@access_control(has_perm_administration)
def get_all(user):
    """ Return all blobs.

    Args:

    Returns:
        List of Blob instances.

    """
    return Blob.get_all()


def get_all_by_user(user):
    """ Return all blobs by user.

    Args:
        user: User

    Returns:
        List of Blob instances for the given user id.

    """
    return Blob.get_all_by_user_id(str(user.id))


@access_control(can_read_or_write_in_workspace)
def get_all_by_workspace(workspace, user):
    """ Get all data that belong to the workspace.

    Args:
        workspace:

    Returns:

    """
    return Blob.get_all_by_workspace(workspace)


@access_control(can_change_owner)
def change_owner(blob, new_user, user):
    """ Change blob's owner.

    Args:
        blob:
        user:
        new_user:

    Returns:
    """
    # FIXME: user can transfer data to anybody, too permissive
    blob.user_id = str(new_user.id)
    blob.save()



""" Url router for the administration site
"""
from django.contrib import admin
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.sites.models import Site
from django.urls import re_path
from django.urls import reverse_lazy
from django.views.generic.base import RedirectView

import core_main_app.components.web_page_login.api as login_page_api
from core_main_app.commons.enums import WEB_PAGE_TYPES
from core_main_app.utils.rendering import admin_render
from core_main_app.views.admin import views as admin_views, ajax as admin_ajax
from core_main_app.views.admin.views import WebPageView
from core_main_app.views.common import views as common_views

admin_urls = [
    re_path(
        r"^login/message/$",
        staff_member_required(
            WebPageView.as_view(
                api=login_page_api,
                get_redirect="core_main_app/admin/web_page/login_page.html",
                post_redirect="admin:core_main_app_login_page",
                web_page_type=WEB_PAGE_TYPES["login"],
            )
        ),
        name="core_main_app_login_page",
    ),
    re_path(r"^login", RedirectView.as_view(url=reverse_lazy("core_main_app_login"))),
    re_path(r"^logout", RedirectView.as_view(url=reverse_lazy("core_main_app_logout"))),
    re_path(
        r"^data$",
        staff_member_required(
            common_views.ViewData.as_view(
                administration=True, template="core_main_app/admin/data/view_data.html"
            )
        ),
        name="core_main_app_data_detail",
    ),
    re_path(
        r"^templates$", admin_views.manage_templates, name="core_main_app_templates"
    ),
    re_path(
        r"^template/upload$",
        admin_views.upload_template,
        name="core_main_app_upload_template",
    ),
    re_path(
        r"^template/upload/(?P<version_manager_id>\w+)",
        admin_views.upload_template_version,
        name="core_main_app_upload_template_version",
    ),
    re_path(
        r"^template/versions/(?P<version_manager_id>\w+)",
        admin_views.manage_template_versions,
        name="core_main_app_manage_template_versions",
    ),
    re_path(
        r"^template/xslt/(?P<template_id>\w+)",
        staff_member_required(
            common_views.TemplateXSLRenderingView.as_view(
                rendering=admin_render,
                template_name="core_main_app/admin/templates_xslt/main.html",
                save_redirect="admin:core_main_app_manage_template_versions",
                back_to_url="admin:core_main_app_manage_template_versions",
            )
        ),
        name="core_main_app_template_xslt",
    ),
    re_path(r"^dashboard$", admin_views.admin_home, name="core_main_app_admin_home"),
    re_path(
        r"^template/resolve-dependencies",
        admin_ajax.resolve_dependencies,
        name="core_main_app_resolve_dependencies",
    ),
    re_path(
        r"^xslt/(?P<pk>[\w-]+)/edit/$",
        staff_member_required(admin_ajax.EditXSLTView.as_view()),
        name="core_main_app_edit_xslt",
    ),
    re_path(
        r"^xslt$",
        staff_member_required(admin_views.XSLTView.as_view()),
        name="core_main_app_xslt",
    ),
    re_path(
        r"^xslt/upload$",
        staff_member_required(admin_views.UploadXSLTView.as_view()),
        name="core_main_app_upload_xslt",
    ),
    re_path(
        r"^xslt/(?P<pk>[\w-]+)/delete/$",
        staff_member_required(admin_ajax.DeleteXSLTView.as_view()),
        name="core_main_app_delete_xslt",
    ),
    re_path(
        r"^edit-rights/(?P<workspace_id>\w+)$",
        staff_member_required(
            common_views.EditWorkspaceRights.as_view(
                administration=True,
                template="core_main_app/admin/workspaces/edit_rights.html",
            )
        ),
        name="core_main_edit_rights_workspace",
    ),
]

urls = admin.site.get_urls()
admin.site.get_urls = lambda: admin_urls + urls

# Admin part for the Site model is not useful in this application
admin.site.unregister(Site)



""" Unit Test Data
"""
from bson.objectid import ObjectId

from core_main_app.commons import exceptions
from core_main_app.components.data.models import Data
from core_main_app.components.data import api as data_api
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.integration_tests.integration_base_test_case import MongoIntegrationBaseTestCase
from core_main_app.settings import DATA_SORTING_FIELDS
from tests.components.data.fixtures.fixtures import DataFixtures, AccessControlDataFixture

fixture_data = DataFixtures()
access_control_data_fixture = AccessControlDataFixture()


class TestDataGetAll(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_data_get_all_return_collection_of_data(self):
        # Act
        result = Data.get_all(DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_data_get_all_return_objects_data_in_collection(self):
        # Act
        result = Data.get_all(DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(len(self.fixture.data_collection) == result.count())

    def test_data_get_all_ordering(self):
        # Arrange
        access_control_data_fixture.insert_data()
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all(ascending_order_by_field)
        descending_result = Data.get_all(descending_order_by_field)
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(ascending_result.all()[i].title == descending_result.all()[len(ascending_result) - i - 1].title)

    def test_data_get_all_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        # Act
        ascending_result = Data.get_all(ascending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_2.title == ascending_result.all()[1].title)

    def test_data_get_all_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        # Act
        descending_result = Data.get_all(descending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_2.title == descending_result.all()[len(descending_result)-2].title)
        self.assertTrue(self.fixture.data_1.title == descending_result.all()[len(descending_result)-1].title)

    def test_data_get_all_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+title", "+user_id"]
        descending_order_by_multi_field = ["+title", "-user_id"]
        # Act
        ascending_result = Data.get_all(ascending_order_by_multi_field)
        descending_result = Data.get_all(descending_order_by_multi_field)
        # Assert
        self.assertEqual(self.fixture.data_4.user_id, ascending_result.all()[4].user_id)
        self.assertEqual(self.fixture.data_5.user_id, ascending_result.all()[3].user_id)

        self.assertEqual(self.fixture.data_4.user_id, descending_result.all()[3].user_id)
        self.assertEqual(self.fixture.data_5.user_id, descending_result.all()[4].user_id)

    def test_data_get_all_api_without_sorting_param_use_default_data_sorting_setting(self):
        # Arrange
        mock_user = _create_user('1', is_superuser=True)
        # Act
        data = data_api.get_all(mock_user)
        # Assert
        self.assertListEqual(list(data), self.fixture.data_collection)


class TestDataGetAllExcept(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_data_get_all_except_return_collection_of_data(self):
        # Act
        db_content = Data.get_all(DATA_SORTING_FIELDS)
        excluded_id_list = [str(db_content[0].pk)]

        result = Data.get_all_except([], excluded_id_list)
        # Assert
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_data_get_all_return_objects_data_in_collection(self):
        # Act
        db_content = Data.get_all(DATA_SORTING_FIELDS)
        excluded_id_list = [str(db_content[0].pk)]

        result = Data.get_all_except([], excluded_id_list)
        # Assert
        self.assertTrue(result.count() == len(self.fixture.data_collection) - len(excluded_id_list))

    def test_data_get_all_except_empty_list_return_collection_of_data(self):
        # Act
        result = Data.get_all_except(DATA_SORTING_FIELDS, id_list=[])
        # Assert
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_data_get_all_except_empty_list_return_objects_data_in_collection(self):
        # Act
        result = Data.get_all_except(DATA_SORTING_FIELDS, id_list=[])
        # Assert
        self.assertTrue(result.count() == len(self.fixture.data_collection))

    def test_data_get_all_except_inexistant_id_return_collection_of_data(self):
        # Act
        object_id_list = [str(data.pk) for data in Data.get_all(DATA_SORTING_FIELDS)]
        inexistant_object_id = str(ObjectId())

        # If the generated object id correspond to one in DB we generate another one
        while inexistant_object_id in object_id_list:
            inexistant_object_id = str(ObjectId)

        excluded_id_list = [inexistant_object_id]

        result = Data.get_all_except([], excluded_id_list)
        # Assert
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_data_get_all_except_inexistant_id_return_objects_data_in_collection(self):
        # Act
        object_id_list = [str(data.pk) for data in Data.get_all(DATA_SORTING_FIELDS)]
        inexistant_object_id = str(ObjectId())

        # If the generated object id correspond to one in DB we generate another one
        while inexistant_object_id in object_id_list:
            inexistant_object_id = str(ObjectId)

        excluded_id_list = [inexistant_object_id]

        result = Data.get_all_except([], excluded_id_list)
        # Assert
        self.assertTrue(result.count() == len(self.fixture.data_collection))

    def test_data_get_all_except_no_params_return_collection_of_data(self):
        # Act
        result = Data.get_all_except(DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_data_get_all_except_no_params_return_objects_data_in_collection(self):
        # Act
        result = Data.get_all_except(DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(result.count() == len(self.fixture.data_collection))

    def test_get_all_except_data_ordering(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_except(ascending_order_by_field)
        descending_result = Data.get_all_except(descending_order_by_field)
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(ascending_result.all()[i].title == descending_result.all()[len(ascending_result) - i - 1].title)

    def test_get_all_except_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        # Act
        ascending_result = Data.get_all_except(ascending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_2.title == ascending_result.all()[1].title)

    def test_get_all_except_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        # Act
        descending_result = Data.get_all_except(descending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_2.title == descending_result.all()[len(descending_result)-2].title)
        self.assertTrue(self.fixture.data_1.title == descending_result.all()[len(descending_result)-1].title)

    def test_get_all_except_data_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+title", "+user_id"]
        descending_order_by_multi_field = ["+title", "-user_id"]
        # Act
        ascending_result = Data.get_all_except(ascending_order_by_multi_field)
        descending_result = Data.get_all_except(descending_order_by_multi_field)
        # Assert
        self.assertEqual(self.fixture.data_4.user_id, ascending_result.all()[4].user_id)
        self.assertEqual(self.fixture.data_5.user_id, ascending_result.all()[3].user_id)

        self.assertEqual(self.fixture.data_4.user_id, descending_result.all()[3].user_id)
        self.assertEqual(self.fixture.data_5.user_id, descending_result.all()[4].user_id)


class TestDataGetById(MongoIntegrationBaseTestCase):

    fixture = fixture_data

    def test_data_get_by_id_raises_api_error_if_not_found(self):
        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            Data.get_by_id(ObjectId())

    def test_data_get_by_id_return_data_if_found(self):
        # Act
        result = Data.get_by_id(self.fixture.data_1.id)
        # Assert
        self.assertEqual(result, self.fixture.data_1)


class TestDataGetAllByUserId(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_data_get_all_by_user_id_return_collection_of_data_from_user(self):
        # Arrange
        user_id = 1
        # Act
        result = Data.get_all_by_user_id(user_id, DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(all(item.user_id == str(user_id) for item in result))

    def test_data_get_all_by_user_id_return_empty_collection_of_data_from_user_does_not_exist(self):
        # Arrange
        user_id = 800
        # Act
        result = Data.get_all_by_user_id(user_id, DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(result.count() == 0)

    def test_get_all_by_user_id_data_ordering(self):
        # Arrange
        user_id = 1
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_by_user_id(user_id, ascending_order_by_field)
        descending_result = Data.get_all_by_user_id(user_id, descending_order_by_field)
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(ascending_result.all()[i].title == descending_result.all()[len(ascending_result) - i - 1].title)

    def test_get_all_by_user_id_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        user_id = 1
        # Act
        ascending_result = Data.get_all_by_user_id(user_id, ascending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_3.title == ascending_result.all()[1].title)

    def test_get_all_by_user_id_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        user_id = 1
        # Act
        descending_result = Data.get_all_by_user_id(user_id, descending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_3.title == descending_result.all()[len(descending_result)-2].title)
        self.assertTrue(self.fixture.data_1.title == descending_result.all()[len(descending_result)-1].title)

    def test_get_all_by_user_id_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+user_id", "+title"]
        descending_order_by_multi_field = ["+user_id", "-title"]
        # Act
        ascending_result = Data.get_all_by_user_id(1, ascending_order_by_multi_field)
        descending_result = Data.get_all_by_user_id(1, descending_order_by_multi_field)
        # Assert
        self.assertEqual(self.fixture.data_1.title, ascending_result.all()[0].title)
        self.assertEqual(self.fixture.data_3.title, ascending_result.all()[1].title)

        self.assertEqual(self.fixture.data_3.title, descending_result.all()[1].title)
        self.assertEqual(self.fixture.data_1.title, descending_result.all()[2].title)

    def test_data_get_all_api_without_sorting_param_use_default_data_sorting_setting(self):
        # Arrange
        mock_user = _create_user('1', is_superuser=True)
        # Act
        data = data_api.get_all_by_user(mock_user)
        # Assert
        self.assertListEqual(list(data), [self.fixture.data_1, self.fixture.data_3, self.fixture.data_5])


class TestDataGetAllExceptUserId(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_data_get_all_except_user_id_return_collection_of_data_where_user_is_not_owner(self):
        # Arrange
        user_id = 1
        # Act
        result = Data.get_all_except_user_id(user_id, DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(item.user_id != user_id for item in result)

    def test_data_get_all_by_user_id_return_full_collection_of_data_from_user_does_not_exist(self):
        # Arrange
        user_id = 800
        # Act
        result = Data.get_all_except_user_id(user_id, DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(result.count() > 0)

    def test_get_all_except_user_id_data_ordering(self):
        # Arrange
        user_id = 1
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_except_user_id(user_id, ascending_order_by_field)
        descending_result = Data.get_all_except_user_id(user_id, descending_order_by_field)
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(ascending_result.all()[i].title == descending_result.all()[len(ascending_result) - i - 1].title)

    def test_get_all_except_user_id_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        user_id = 2
        # Act
        ascending_result = Data.get_all_except_user_id(user_id, ascending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_3.title == ascending_result.all()[1].title)

    def test_get_all_except_user_id_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        user_id = 2
        # Act
        descending_result = Data.get_all_except_user_id(user_id, descending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_3.title == descending_result.all()[len(descending_result)-2].title)
        self.assertTrue(self.fixture.data_1.title == descending_result.all()[len(descending_result)-1].title)

    def test_get_all_except_user_id_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+title", "+user_id"]
        descending_order_by_multi_field = ["+title", "-user_id"]
        # Act
        ascending_result = Data.get_all_except_user_id(3, ascending_order_by_multi_field)
        descending_result = Data.get_all_except_user_id(3, descending_order_by_multi_field)
        # Assert
        self.assertEqual(self.fixture.data_4.user_id, ascending_result.all()[4].user_id)
        self.assertEqual(self.fixture.data_5.user_id, ascending_result.all()[3].user_id)

        self.assertEqual(self.fixture.data_4.user_id, descending_result.all()[3].user_id)
        self.assertEqual(self.fixture.data_5.user_id, descending_result.all()[4].user_id)

    def test_data_get_all_except_user_api_without_sorting_param_use_default_data_sorting_setting(self):
        # Arrange
        mock_user = _create_user('1', is_superuser=True)
        # Act
        data = data_api.get_all_except_user(mock_user)
        #Assert
        self.assertListEqual(list(data), [self.fixture.data_2, self.fixture.data_4])


class TestExecuteQuery(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_execute_query_data_ordering(self):
        # Arrange
        query = {}
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.execute_query(query, ascending_order_by_field)
        descending_result = Data.execute_query(query, descending_order_by_field)
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(ascending_result.all()[i].title == descending_result.all()[len(ascending_result) - i - 1].title)

    def test_execute_query_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        query = {}
        # Act
        ascending_result = Data.execute_query(query, ascending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_2.title == ascending_result.all()[1].title)

    def test_execute_query_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        query = {}
        # Act
        descending_result = Data.execute_query(query, descending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_2.title == descending_result.all()[len(descending_result)-2].title)
        self.assertTrue(self.fixture.data_1.title == descending_result.all()[len(descending_result)-1].title)

    def test_execute_query_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+title", "+user_id"]
        descending_order_by_multi_field = ["+title", "-user_id"]
        query = {}
        # Act
        ascending_result = Data.execute_query(query, ascending_order_by_multi_field)
        descending_result = Data.execute_query(query, descending_order_by_multi_field)
        # Assert
        self.assertEqual(self.fixture.data_4.user_id, ascending_result.all()[4].user_id)
        self.assertEqual(self.fixture.data_5.user_id, ascending_result.all()[3].user_id)

        self.assertEqual(self.fixture.data_4.user_id, descending_result.all()[3].user_id)
        self.assertEqual(self.fixture.data_5.user_id, descending_result.all()[4].user_id)

    def test_data_execute_query_api_without_sorting_param_use_default_data_sorting_setting(self):
        # Arrange
        mock_user = _create_user('1', is_superuser=True)
        # Act
        data = data_api.execute_query({}, mock_user)
        #Assert
        self.assertListEqual(list(data), self.fixture.data_collection)


class TestGetAllByWorkspace(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_get_all_by_workspace_data_ordering(self):
        # Arrange
        workspace = self.fixture.workspace_1.id
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_by_workspace(workspace, ascending_order_by_field)
        descending_result = Data.get_all_by_workspace(workspace, descending_order_by_field)
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(ascending_result.all()[i].title == descending_result.all()[len(ascending_result) - i - 1].title)

    def test_get_all_by_workspace_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        workspace = self.fixture.workspace_1.id
        # Act
        ascending_result = Data.get_all_by_workspace(workspace, ascending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_3.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_5.title == ascending_result.all()[1].title)

    def test_get_all_by_workspace_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        workspace = self.fixture.workspace_1.id
        # Act
        descending_result = Data.get_all_by_workspace(workspace, descending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_5.title == descending_result.all()[len(descending_result)-2].title)
        self.assertTrue(self.fixture.data_3.title == descending_result.all()[len(descending_result)-1].title)

    def test_get_all_by_workspace_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+workspace", "+title"]
        descending_order_by_multi_field = ["+workspace", "-title"]
        workspace = self.fixture.workspace_1.id
        # Act
        ascending_result = Data.get_all_by_workspace(workspace, ascending_order_by_multi_field)
        descending_result = Data.get_all_by_workspace(workspace, descending_order_by_multi_field)
        # Assert
        self.assertEqual(self.fixture.data_3.user_id, ascending_result.all()[0].user_id)
        self.assertEqual(self.fixture.data_5.user_id, ascending_result.all()[1].user_id)

        self.assertEqual(self.fixture.data_3.user_id, descending_result.all()[1].user_id)
        self.assertEqual(self.fixture.data_5.user_id, descending_result.all()[0].user_id)

    def test_data_get_all_by_workspace_api_without_sorting_param_use_default_data_sorting_setting(self):
        # Arrange
        mock_user = _create_user('1', is_superuser=True)
        workspace = self.fixture.workspace_1.id
        # Act
        data = data_api.get_all_by_workspace(workspace, mock_user)
        # Assert
        self.assertListEqual(list(data), [self.fixture.data_3, self.fixture.data_5])


class TestGetAllByListTemplate(MongoIntegrationBaseTestCase):
    fixture = access_control_data_fixture

    def test_returns_data_object(self):
        result = Data.get_all_by_list_workspace(
            [self.fixture.workspace_1.id, self.fixture.workspace_2.id],
            DATA_SORTING_FIELDS
        )
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_returns_correct_count(self):
        result = Data.get_all_by_list_workspace(
            [self.fixture.workspace_1.id, self.fixture.workspace_2.id],
            DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 3)

    def test_none_returns_data_object(self):
        result = Data.get_all_by_list_workspace([None], DATA_SORTING_FIELDS)
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_none_returns_correct_count(self):
        result = Data.get_all_by_list_workspace([None], DATA_SORTING_FIELDS)
        self.assertEqual(len(result), 2)

    def test_empty_list_returns_no_data(self):
        result = Data.get_all_by_list_workspace([], DATA_SORTING_FIELDS)
        self.assertEqual(len(result), 0)

    def test_invalid_workspace_returns_no_data(self):
        result = Data.get_all_by_list_workspace([ObjectId()], DATA_SORTING_FIELDS)
        self.assertEqual(len(result), 0)


class TestGetAllByTemplatesAndWorkspaces(MongoIntegrationBaseTestCase):
    fixture = access_control_data_fixture

    def test_returns_data_object(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id],
            [self.fixture.workspace_1.id, self.fixture.workspace_2.id],
            DATA_SORTING_FIELDS
        )
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_returns_correct_count(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id],
            [self.fixture.workspace_1.id, self.fixture.workspace_2.id],
            DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 3)

    def test_null_workspace_returns_data_object(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id],
            [None],
            DATA_SORTING_FIELDS
        )
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_null_workspace_returns_correct_count(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id],
            [None],
            DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 2)

    def test_empty_workspaces_returns_no_data(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id],
            [],
            DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 0)

    def test_empty_templates_returns_no_data(self):
        result = Data.get_all_by_templates_and_workspaces(
            [],
            [None],
            DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 0)

    def test_invalid_workspace_returns_no_data(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id],
            [ObjectId()],
            DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 0)

    def test_invalid_template_returns_no_data(self):
        result = Data.get_all_by_templates_and_workspaces(
            [ObjectId()],
            [None],
            DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 0)


class TestGetAllAccessibleByUser(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_data_get_all_accessible_by_user_api_without_sorting_param_use_default_data_sorting_setting(self):
        # Arrange
        mock_user = _create_user('1', is_superuser=True)
        # Act
        data = data_api.get_all_accessible_by_user(mock_user)
        # Assert
        self.assertListEqual(list(data), [self.fixture.data_1, self.fixture.data_3, self.fixture.data_5])


class TestGetAllByUserAndWorkspace(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_get_all_data_from_user_and_from_workspace_for_user_within_workspace(self):
        # Arrange
        mock_user = create_mock_user('1', is_superuser=False)
        # Act
        data = Data.get_all_by_user_and_workspace(mock_user.id, [self.fixture.workspace_1,
                                                                 self.fixture.workspace_2], ['+title'])
        # Assert
        self.assertListEqual(list(data), [self.fixture.data_1, self.fixture.data_3,
                                          self.fixture.data_4, self.fixture.data_5])

    def test_get_all_data_from_user_and_from_workspace_data_ordering(self):
        # Arrange
        mock_user = create_mock_user('1', is_superuser=False)
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_by_user_and_workspace(mock_user.id, [self.fixture.workspace_1], ascending_order_by_field)
        descending_result = Data.get_all_by_user_and_workspace(mock_user.id, [self.fixture.workspace_1], descending_order_by_field)
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(ascending_result.all()[i].title == descending_result.all()[len(ascending_result) - i - 1].title)

    def test_get_all_data_from_user_and_from_workspace_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        mock_user = create_mock_user('1', is_superuser=False)
        # Act
        ascending_result = Data.get_all_by_user_and_workspace(mock_user.id, [self.fixture.workspace_1], ascending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_3.title == ascending_result.all()[1].title)

    def test_get_all_data_from_user_and_from_workspace_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        mock_user = create_mock_user('1', is_superuser=False)
        # Act
        descending_result = Data.get_all_by_user_and_workspace(mock_user.id, [self.fixture.workspace_1], descending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_3.title == descending_result.all()[len(descending_result)-2].title)
        self.assertTrue(self.fixture.data_1.title == descending_result.all()[len(descending_result)-1].title)

    def test_get_all_data_from_user_and_from_workspace_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+template", "+title"]
        descending_order_by_multi_field = ["+template", "-title"]
        mock_user = create_mock_user('1', is_superuser=False)
        # Act
        ascending_result = Data.get_all_by_user_and_workspace(mock_user.id, [self.fixture.workspace_1], ascending_order_by_multi_field)
        descending_result = Data.get_all_by_user_and_workspace(mock_user.id, [self.fixture.workspace_1], descending_order_by_multi_field)
        # Assert
        self.assertEqual(self.fixture.data_1.title, ascending_result.all()[0].title)
        self.assertEqual(self.fixture.data_3.user_id, ascending_result.all()[1].user_id)

        self.assertEqual(self.fixture.data_3.user_id, descending_result.all()[1].user_id)
        self.assertEqual(self.fixture.data_1.user_id, descending_result.all()[2].user_id)


def _create_user(user_id, is_superuser=False):
    return create_mock_user(user_id, is_superuser=is_superuser)



""" Authentication tests for Data REST API
"""
from django.test import SimpleTestCase
from mock import Mock
from mock.mock import patch
from rest_framework import status
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK

from core_main_app.components.data import api as data_api
from core_main_app.components.data.models import Data
from core_main_app.components.workspace import api as workspace_api
from core_main_app.rest.data import views as data_rest_views
from core_main_app.rest.data.abstract_views import AbstractExecuteLocalQueryView
from core_main_app.rest.data.serializers import DataSerializer, DataWithTemplateInfoSerializer
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestDataListPostPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(
            data_rest_views.DataList.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(DataSerializer, "is_valid")
    @patch.object(DataSerializer, "save")
    @patch.object(DataSerializer, "data")
    def test_authenticated_returns_http_201(self, data_serializer_data, data_serializer_save, data_serializer_valid):
        data_serializer_valid.return_value = True
        data_serializer_save.return_value = None
        data_serializer_data.return_value = {}

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_post(
            data_rest_views.DataList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(DataSerializer, "is_valid")
    @patch.object(DataSerializer, "save")
    @patch.object(DataSerializer, "data")
    def test_staff_returns_http_201(self, data_serializer_data, data_serializer_save, data_serializer_valid):
        data_serializer_valid.return_value = True
        data_serializer_save.return_value = None
        data_serializer_data.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_post(
            data_rest_views.DataList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestDataListGetPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            data_rest_views.DataList.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(Data, "get_all_by_user_id")
    def test_authenticated_returns_http_200(self, data_get_all_by_user):
        data_get_all_by_user.return_value = {}

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            data_rest_views.DataList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Data, "get_all_by_user_id")
    def test_staff_returns_http_200(self, data_get_all_by_user):
        data_get_all_by_user.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.DataList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataListByWorkspaceGetPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            data_rest_views.DataListByWorkspace.as_view(),
            None,
            param={"workspace_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "get_all_by_workspace")
    def test_authenticated_returns_http_200(self, data_get_all_by_workspace):
        data_get_all_by_workspace.return_value = {}

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            data_rest_views.DataListByWorkspace.as_view(),
            mock_user,
            param={"workspace_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "get_all_by_workspace")
    def test_staff_returns_http_200(self, data_get_all_by_workspace):
        data_get_all_by_workspace.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.DataListByWorkspace.as_view(),
            mock_user,
            param={"workspace_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataDetailGetPermissions(SimpleTestCase):
    @patch.object(DataSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_anonymous_returns_http_200(self, mock_data_api_get_by_id, mock_data_serializer_data):
        mock_data_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        response = RequestMock.do_request_get(
            data_rest_views.DataDetail.as_view(),
            None,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(self, mock_data_api_get_by_id, mock_data_serializer_data):
        mock_data_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        mock_user = create_mock_user('1')
        response = RequestMock.do_request_get(
            data_rest_views.DataDetail.as_view(),
            mock_user,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(self, mock_data_api_get_by_id, mock_data_serializer_data):
        mock_data_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.DataDetail.as_view(),
            mock_user,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataDetailDeletePermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_delete(
            data_rest_views.DataDetail.as_view(),
            None,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "delete")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_204(self, mock_data_api_get_by_id, mock_data_api_delete):
        mock_data_api_get_by_id.return_value = None
        mock_data_api_delete.return_value = None

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_delete(
            data_rest_views.DataDetail.as_view(),
            mock_user,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(data_api, "delete")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_204(self, mock_data_api_get_by_id, mock_data_api_delete):
        mock_data_api_get_by_id.return_value = None
        mock_data_api_delete.return_value = None

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_delete(
            data_rest_views.DataDetail.as_view(),
            mock_user,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestDataDetailPatchPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            data_rest_views.DataDetail.as_view(),
            None,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(DataSerializer, "data")
    @patch.object(DataSerializer, "save")
    @patch.object(DataSerializer, "is_valid")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(self, mock_data_api_get_by_id, mock_data_serializer_is_valid,
                                            mock_data_serializer_save, mock_data_serializer_data):
        mock_data_api_get_by_id.return_value = []
        mock_data_serializer_is_valid.return_value = True
        mock_data_serializer_save.return_valie = None
        mock_data_serializer_data.return_value = []

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            data_rest_views.DataDetail.as_view(),
            mock_user,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataSerializer, "data")
    @patch.object(DataSerializer, "save")
    @patch.object(DataSerializer, "is_valid")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(self, mock_data_api_get_by_id, mock_data_serializer_is_valid,
                                    mock_data_serializer_save, mock_data_serializer_data):
        mock_data_api_get_by_id.return_value = []
        mock_data_serializer_is_valid.return_value = True
        mock_data_serializer_save.return_valie = None
        mock_data_serializer_data.return_value = []

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_patch(
            data_rest_views.DataDetail.as_view(),
            mock_user,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataDownloadGetPermissions(SimpleTestCase):

    @patch.object(data_api, "get_by_id")
    def test_anonymous_returns_http_200(self, mock_data_api_get_by_id):
        data_object = Mock()
        data_object.xml_content = "<tag attr='attr_val'>tag_val</tag>"
        data_object.title = "mock.xml"

        mock_data_api_get_by_id.return_value = data_object

        response = RequestMock.do_request_get(
            data_rest_views.DataDownload.as_view(),
            None,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(self, mock_data_api_get_by_id):
        data_object = Mock()
        data_object.xml_content = "<tag attr='attr_val'>tag_val</tag>"
        data_object.title = "mock.xml"

        mock_data_api_get_by_id.return_value = data_object

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            data_rest_views.DataDownload.as_view(),
            mock_user,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(self, mock_data_api_get_by_id):
        data_object = Mock()
        data_object.xml_content = "<tag attr='attr_val'>tag_val</tag>"
        data_object.title = "mock.xml"

        mock_data_api_get_by_id.return_value = data_object

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.DataDownload.as_view(),
            mock_user,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataGetFullGetPermissions(SimpleTestCase):
    @patch.object(DataWithTemplateInfoSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_anonymous_returns_http_200(self, mock_data_api_get_by_id, mock_data_with_template_serializer_data):
        mock_data_api_get_by_id.return_value = None
        mock_data_with_template_serializer_data.return_value = []

        response = RequestMock.do_request_get(
            data_rest_views.get_by_id_with_template_info,
            None,
            data={"id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataWithTemplateInfoSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(self, mock_data_api_get_by_id, mock_data_with_template_serializer_data):
        mock_data_api_get_by_id.return_value = None
        mock_data_with_template_serializer_data.return_value = []

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            data_rest_views.get_by_id_with_template_info,
            mock_user,
            data={"id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataWithTemplateInfoSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(self, mock_data_api_get_by_id, mock_data_with_template_serializer_data):
        mock_data_api_get_by_id.return_value = None
        mock_data_with_template_serializer_data.return_value = []

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.get_by_id_with_template_info,
            mock_user,
            data={"id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataLocalQueryGetPermissions(SimpleTestCase):
    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_anonymous_returns_http_200(self, mock_abstract_local_query_execute_query):
        mock_abstract_local_query_execute_query.return_value = Response(status=HTTP_200_OK)

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            None,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_authenticated_returns_http_200(self, mock_abstract_local_query_execute_query):
        mock_abstract_local_query_execute_query.return_value = Response(status=HTTP_200_OK)

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_staff_returns_http_200(self, mock_abstract_local_query_execute_query):
        mock_abstract_local_query_execute_query.return_value = Response(status=HTTP_200_OK)

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataLocalQueryPostPermissions(SimpleTestCase):
    @patch.object(data_api, "execute_query")
    def test_anonymous_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            None,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "execute_query")
    def test_authenticated_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "execute_query")
    def test_staff_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataKeywordQueryGetPermissions(SimpleTestCase):

    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_anonymous_returns_http_200(self, mock_abstract_local_query_execute_query):
        mock_abstract_local_query_execute_query.return_value = Response(status=HTTP_200_OK)

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            None,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_authenticated_returns_http_200(self, mock_abstract_local_query_execute_query):
        mock_abstract_local_query_execute_query.return_value = Response(status=HTTP_200_OK)

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_staff_returns_http_200(self, mock_abstract_local_query_execute_query):
        mock_abstract_local_query_execute_query.return_value = Response(status=HTTP_200_OK)

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataKeywordQueryPostPermissions(SimpleTestCase):
    @patch.object(data_api, "execute_query")
    def test_anonymous_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalKeywordQueryView.as_view(),
            None,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "execute_query")
    def test_authenticated_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalKeywordQueryView.as_view(),
            mock_user,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "execute_query")
    def test_staff_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalKeywordQueryView.as_view(),
            mock_user,
            data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataAssignPatchPermissions(SimpleTestCase):
    @patch.object(data_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_anonymous_returns_http_403(self, mock_data_api_get_by_id, mock_workspace_api_get_by_id,
                                        mock_data_api_assign):
        mock_data_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_data_api_assign.return_value = None

        response = RequestMock.do_request_patch(
            data_rest_views.DataAssign.as_view(),
            None,
            param={"pk": 0, "workspace_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(self, mock_data_api_get_by_id, mock_workspace_api_get_by_id,
                                            mock_data_api_assign):
        mock_data_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_data_api_assign.return_value = None

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            data_rest_views.DataAssign.as_view(),
            mock_user,
            param={"pk": 0, "workspace_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(self, mock_data_api_get_by_id, mock_workspace_api_get_by_id,
                                    mock_data_api_assign):
        mock_data_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_data_api_assign.return_value = None

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_patch(
            data_rest_views.DataAssign.as_view(),
            mock_user,
            param={"pk": 0, "workspace_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataChangeOwnerPatchPermissions(SimpleTestCase):

    @patch.object(data_api, "change_owner")
    @patch('core_main_app.components.user.api.get_user_by_id')
    @patch.object(Data, 'get_by_id')
    def test_anonymous_returns_http_403(self,
                                        mock_data_api_get_by_id,
                                        mock_user_api_get_by_id,
                                        mock_data_api_change_owner):
        mock_data_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_data_api_change_owner.return_value = None

        response = RequestMock.do_request_patch(
            data_rest_views.DataChangeOwner.as_view(),
            None,
            param={"pk": 0, "user_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "change_owner")
    @patch('core_main_app.components.user.api.get_user_by_id')
    @patch.object(Data, 'get_by_id')
    def test_authenticated_returns_http_403(self,
                                            mock_data_api_get_by_id,
                                            mock_user_api_get_by_id,
                                            mock_data_api_change_owner):
        mock_data_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_data_api_change_owner.return_value = None

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            data_rest_views.DataChangeOwner.as_view(),
            mock_user,
            param={"pk": 0, "user_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "change_owner")
    @patch('core_main_app.components.user.api.get_user_by_id')
    @patch.object(Data, 'get_by_id')
    def test_staff_returns_http_200(self,
                                    mock_data_api_get_by_id,
                                    mock_user_api_get_by_id,
                                    mock_data_api_change_owner):
        # Arrange
        # is_staff to access the view
        # is_superuser to be able to change the owner
        user_request = create_mock_user("1", is_staff=True, is_superuser=True)
        mock_data_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_data_api_change_owner.return_value = None

        # Act
        response = RequestMock.do_request_patch(
            data_rest_views.DataChangeOwner.as_view(),
            user_request,
            param={"pk": 0, "user_id": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataPermissions(SimpleTestCase):

    @patch.object(Data, 'get_by_id')
    def test_superuser_returns_http_200(self, get_by_id):
        mock_user = create_mock_user('1', is_superuser=True)
        mock_data = Data(user_id='1')
        get_by_id.return_value = mock_data

        response = RequestMock.do_request_get(data_rest_views.DataPermissions.as_view(),
                                              mock_user,
                                              data={'ids': f'["{mock_data.id}"]'})

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Data, 'get_by_id')
    def test_authenticated_returns_http_200(self, get_by_id):
        mock_user = create_mock_user('1')
        mock_data = Data(user_id='1')
        get_by_id.return_value = mock_data

        response = RequestMock.do_request_get(data_rest_views.DataPermissions.as_view(),
                                              mock_user,
                                              data={'ids': f'["{mock_data.id}"]'})

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Data, 'get_by_id')
    def test_staff_returns_http_200(self, get_by_id):
        mock_user = create_mock_user('1', is_staff=True)
        mock_data = Data(user_id='1')
        get_by_id.return_value = mock_data

        response = RequestMock.do_request_get(data_rest_views.DataPermissions.as_view(),
                                              mock_user,
                                              data={'ids': f'["{mock_data.id}"]'})

        self.assertEqual(response.status_code, status.HTTP_200_OK)


    @patch.object(Data, 'get_by_id')
    def test_get_returns_200_for_anonymous(self, get_by_id):
        # Arrange
        mock_user = create_mock_user('999', is_anonymous=True)
        mock_data = Data(user_id='1')
        get_by_id.return_value = mock_data

        # Mock
        response = RequestMock.do_request_get(data_rest_views.DataPermissions.as_view(),
                                              mock_user,
                                              data={'ids': '["1"]'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)